// Copyright IBM Corporation 2000, 2009

/*
<trigger>
	<usage>bsf create_associated_task.js "See CR %problem_number: %problem_synopsis"</usage>
	<version>5.0</version>
	<description>
		提出時または遷移時に新しいタスクを作成して CR に関連付けます。
		タスクは、_CREATE_TASK 属性を遷移リクエストの一部として送信した時にのみ作成されます。通常は、このリクエストは、CCM_TOGGLE タイプの _CREATE_TASK 属性を遷移用のフォームに含めることで行います。こうすれば、ユーザーはチェック・ボックスをクリックするだけで、遷移の最中に新しいタスクを作成するかどうかを選択できます。
	</description>
	<argument1>
		<name>synopsis</name>
		<required>no</required>
		<type>string</type>
		<description>この概要の値が、作成されるタスクに与えられます。
                空白を含む概要の値は、二重引用符で囲んでください。
				概要では、親である CR からの属性を % 記号を使って参照できます。
例えば、デフォルトのタスクの概要値は、"See CR %problem_number: %problem_synopsis" であり、CR の problem_number と problem_synopsis の値を概要に含めることができます。
				存在しない属性を参照した場合は、属性名がそのまま概要に入ります。
		</description>
	</argument1>
	<argument2>
		<name>-releaseattr cr_release_attribute_name</name>
		<required>no</required>
		<type>string</type>
		<description>
			通常は、'release' 属性は CR から新しいタスクにコピーされます。
			このパラメーターを指定すると、-releaseattr target_release と指定した場合のように、CR とは異なる属性値をタスクのリリース値として使用できます。
		</description>
	</argument2>
	<argument3>
		<name>-resolverattr cr_user_attribute</name>
		<required>no</required>
		<type>string</type>
		<description>
			通常は、'resolver' 属性は CR から新しいタスクにコピーされます。このパラメーターを指定すると、-resolverattr enterer と指定した場合のように、CR とは異なる属性値をタスクの解決者として使用できます。
		</description>
	</argument3>
</trigger>
*/

importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

// check if the _CREATE_TASK pseudo-attribute is present.
// If so, we should create a new task.
var createTask = false;
for (var i = 0; i < event.attributes.length; i++)
{
	if (event.attributes[i].name == '_CREATE_TASK')
	{
		createTask = true;
		break;
	}
}

if (createTask)
{
	// Some lifecycles may want to set the task 'resolver' and 'release' from
	// custom attributes on the CR. Check for custom attribute names, or use the default if there are none.
	var options = OptionParser.parse(args);
	var resolverAttrName = options.getOption('-resolverattr') || 'resolver';
	var releaseAttrName = options.getOption('-releaseattr') || 'release';

	// use the given task synopsis (a template), or a default
	var synopsisTemplate = new KeywordString(options.args[0] || 'See CR %problem_number: %problem_synopsis');

	// get all the attribute values we need from the CR that we'll apply to the task.
	// This includes the attributes needed to expanded the attribute keywords in the synopsis.
	var crAttributes = change.collectAttributes(
		event.triggerUser.token,
		event,
		[resolverAttrName, releaseAttrName].concat(synopsisTemplate.getKeywords()));

	var taskSynopsis = synopsisTemplate.replaceKeywords(mapToStringValues(crAttributes));

	var taskAttributes = [
		new Attribute('task_synopsis', taskSynopsis),
		new Attribute('priority', 'medium')
		];
		
	// ensure the inherited attributes are on the CR before we use them.
	// They may not be required for this transition
	var crResolver = crAttributes.get(resolverAttrName);
	var crRelease = crAttributes.get(releaseAttrName);
	if (crResolver && crResolver.stringValue) taskAttributes.push(new Attribute('resolver', crResolver.stringValue));
	if (crRelease && crRelease.stringValue) taskAttributes.push(new Attribute('release', crRelease.stringValue));

	var taskId = change.createAssociatedTask(
		event.triggerUser.token,
		event.objectId,
		taskAttributes);

	log.debug('CR ' + event.objectId + ': associated new task ' + taskId);
}

/**
 * Changes Map<String, Attribute> into a Map<String, String>,
 * using the Attribute string values as the new map's values.
 * The latter form can be passed to replaceKeywords.
 */
function mapToStringValues(attributeMap)
{
	var valueMap = new java.util.HashMap();
	for (var iter = attributeMap.keySet().iterator(); iter.hasNext();)
	{
		var key = iter.next();
		valueMap.put(key, attributeMap.get(key).toString());
	}

	return valueMap;
}
