// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.
/*
<trigger>
	<usage>bsf create_relationship.js relationship -from|-to other_cr_attribute</usage>
	<version>5.0</version>
	<description>
		ある CR の遷移の間に、他の既存の CR との関係を作成します。
		他の既存の CR の番号は、遷移する CR の属性として提出されます。
		他の CR の番号を特定するために使用する属性名が、このスクリプトへの引数となります。
		関係の方向付けは、-from または -to オプションの指定で決定されます。
		これらのオプションには、他の CR を指定します。
例えば、遷移する CR に対して、_DUPLICATE 属性に指定された CR からの "duplicate" という名前の関係を作成するには、スクリプトの引数を "duplicate -from _DUPLICATE" とします。
</description>
	<argument1>
		<name>relationship</name>
		<required>yes</required>
		<type>string</type>
		<description>作成したい関係の名前
		</description>
	</argument1>
	<argument2>
		<name>-from|-to other_cr_attribute</name>
		<required>yes</required>
		<type>string</type>
		<description>
		遷移の間に関係付けを行いたい「他の」CR の番号を指定した属性の名前。この名前が遷移の際に送信されます。
例えば、デフォルトの "Mark as Duplicate" 遷移では、_DUPLICATE 属性が関係付けのために使用されます。
            -to オプションを指定すると、その「他の」CR への関係が作成され、-from オプションを使用すると、その「他の」CR からの関係が作成されます。
			下線で始まる属性名を使用すると、一時的な情報として送信されるため、CR に保存されないことに注意してください。		
		</description>
	</argument2>
</trigger>
*/
importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

var options = OptionParser.parse(args);

if (options.args.length != 1
	|| (options.getOption('-to') == null && options.getOption('-from') == null))
{
	// check for three args, and that the second is one of the known option values
	usage();
}
else
{
	var relationship = options.args[0];
	var reverseDirection = options.getOption('-from');
	var attributeWithOtherCrid = options.getOption('-to') || options.getOption('-from');
	
	if (options.getOption('-to'))
	{
		var fromCrid = event.objectId;
		var toCrid = findOtherCrid(options.getOption('-to'));
	}
	else
	{
		var fromCrid = findOtherCrid(options.getOption('-from'));
		var toCrid = event.objectId;

	}
	
	if (toCrid && fromCrid)
	{
		log.debug('Creating ' + relationship + ' from ' + fromCrid + ' to ' + toCrid);

		try
		{
			change.relateNoSecurity(
				event.triggerUser.token,
				fromCrid,
				toCrid,
				relationship);
		}
		catch (e)
		{
			if (e.javaException)
			{
				event.fail(e.javaException.getMessage());
			}
			else
			{
				throw e;
			}
		}
	}
}

/**
 * Finds the named attribute on the event and returns it string value.
 * This is assumed to be the CRID of the "other" CR involved in this relationship.
 * @param inAttribute name of a modifed attribute that contains the CR ID to relate with.
 * @return the value of the named attribute, or undefined if not found.
 */
function findOtherCrid(inAttribute)
{
	for (var i = 0; i < event.attributes.length; i++)
	{
		// use .equals, since these are Java strings; == would compare their references, not their values.
		if (event.attributes[i].name.equals(inAttribute))
		{
			return event.attributes[i].stringValue;
		}
	}
}

function usage()
{
	log.error('Could not create relationship: missing arguments.');
	log.error('Usage: create_relationship.js relationship -from|-to attribute_containing_crid_to_relate_to');
	log.error('Example: create_relationship.js duplicate -from _DUPLICATE');
}