# <trigger>
#	<usage>perl checkRCRConflicts.pl</usage>
#	<version>4.6</version>
#	<description>このトリガーは、RCR ライフサイクル内の遷移前トリガーとして使用するためのものです。このトリガーは、遷移対象の RCR と、その RCR が遷移しようとしている状態のすべての RCR との間に、競合がないかどうか検査します。例えば、RCR が「rcr_レビュー」から「rcr_承認済み」に遷移しようとしている場合、このトリガーは、遷移対象の RCR と、「rcr_承認済み」状態のすべての RCR との間に、競合がないかどうか検査します。競合が検出された場合、トリガーはエラー・メッセージとともに終了し、遷移が失敗します。このトリガーには、doors_cs_template で定義されたレポートが必要です。
#   </description>
# </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;
use ChangeSynergy::mrcActions;

our $RC_ACTION_ADD		= "Add";
our $RC_ACTION_MOVE		= "Move";
our $RC_ACTION_UNDELETE	= "Undelete";
our $RC_ACTION_MODIFY		= "Modify";
our $RC_ACTION_DELETE		= "Delete";
our $RC_ACTION_PURGE		= "Purge";
#our $RC_ACTION_REMOVE_OUTLINKS = "Remove outlinks";
our $RC_ACTION_CREATE_OUTLINKS = "Create outlinks";

our $MESSAGE_CONFLICT = "Unable to approve RCR due to conflict.";

sub ecpsSplitID($;$;$)
{
	my $strippedReq = $_[0];
	$strippedReq =~ /req/;
	my $rest = $';
	my @components = split(/_/, $rest);
	$_[1] = $components[0];
	$_[2] = $components[1];
}

sub theSameOutlinkTarget($;$;$)
{
	my $linkData = shift;
	my $mid = shift;
	my $oid = shift;
	my @linkParts = split(/\|/, $linkData);
	my $tMid = $linkParts[2];

	if ($tMid eq $mid)
	{
		my $tMajor = $linkParts[3];
		my $tMinor = $linkParts[4];
	
		if ((!defined $tMajor) && (!defined $tMinor))
		{
			my $tOid = $linkParts[6];
		
			if ($tOid eq $oid)
			{
				return 1; #TRUE
			}
		}
	}

	return 0; #FALSE
}

sub checkAttributeModificationConflict($;$)
{
	my $aOrc = shift;
	my $bOrc = shift;

	if (!$aOrc->has_action($RC_ACTION_PURGE))
	{
		if (!$bOrc->has_action($RC_ACTION_PURGE))
		{
			if ($aOrc->has_action($RC_ACTION_MODIFY))
			{
				if ($bOrc->has_action($RC_ACTION_MODIFY))
				{
					ecpsSplitID($aOrc->get_doorsID(), my $aModuleID, my $aObjId);
	
					if ($aObjId ne "0")
					{
						my $aModifiedAttrs = $aOrc->get_modify_attrs();
					
						foreach my $aModifiedAttr (@$aModifiedAttrs)
						{
							if ($bOrc->has_modify_attr($aModifiedAttr))
							{
								my $objPath = $aOrc->get_doors_object_path();
								my $aRcr = $aOrc->get_parent_rcrID();
								my $bRcr = $bOrc->get_parent_rcrID();
								die "Unable to approve RCR " . $aRcr . " due to attribute \"" . $aModifiedAttr . "\" modify conflict on object " . $objPath . " with RCR " . $bRcr . "\n";
							}
						}
					}
				}
			}
		}
	}
}

sub checkPurgeConflict($;$;$)
{
	my $orc = shift;
	my $aRcr = shift;
	my $bRcr = shift;
	my $purgeConflicts = 0; #false

	if (($orc->has_action($RC_ACTION_MODIFY)) || ($orc->has_action($RC_ACTION_CREATE_OUTLINKS)))
	{
		$purgeConflicts = 1; #true
	}
	else
	{
		if (!$orc->has_action($RC_ACTION_DELETE))
		{
			if (($orc->has_action($RC_ACTION_UNDELETE)) || ($orc->has_action($RC_ACTION_MOVE)))
			{
				$purgeConflicts = 1; #true
			}
		}
	}

	if ($purgeConflicts)
	{
		my $objPath = $orc->get_doors_object_path();
		die "Unable to approve RCR $aRcr due to purge conflict on object $objPath with RCR $bRcr\n";
	}
}

sub checkDeleteConflict($;$;$)
{
	my $orc = shift;
	my $aRcr = shift;
	my $bRcr = shift;
	my $deleteConflicts = 0; #false
	
	if ($orc->has_action($RC_ACTION_MODIFY))
	{	
		$deleteConflicts = 1; #true
	}
	elsif ($orc->has_action($RC_ACTION_CREATE_OUTLINKS))
	{
		$deleteConflicts = 1; #true
	}
	elsif ($orc->has_action($RC_ACTION_UNDELETE)) 
	{
		$deleteConflicts = 1; #true
	}
	elsif ($orc->has_action($RC_ACTION_MOVE))
	{
		$deleteConflicts = 1; #true
	}
	
	if ($deleteConflicts)
	{
		my $objPath = $orc->get_doors_object_path();
		die "Unable to approve RCR $aRcr due to delete conflict on object $objPath with RCR $bRcr\n";
	}
}

sub checkMoveConflict($;$)
{
	my $aOrc = shift;
	my $bOrc = shift;
	my $aoBelow;
	my $boBelow;

	if ($aOrc->has_action($RC_ACTION_MOVE))
	{
		if($bOrc->has_action($RC_ACTION_MOVE))
		{
			$aoBelow = $aOrc->get_below();
			$boBelow = $bOrc->get_below();
	
			if ($aoBelow ne $boBelow)
			{
				my $objPath = $aOrc->get_doors_object_path();
				my $aRcr = $aOrc->get_parent_rcrID();
				my $bRcr = $bOrc->get_parent_rcrID();
				die "Unable to approve RCR $aRcr due to move conflict on object $objPath with RCR $bRcr\n";
			}
		}
	}
}

sub checkParentConflict($;$;$)
{
	my $aOrc = shift;
	my $bOrc = shift;
	my $reverseCase = shift;
	my $pOrc = $aOrc;
	my $cOrc = $bOrc;

	if ($reverseCase)
	{
		$pOrc = $bOrc;
		$cOrc = $aOrc;
	}
	ecpsSplitID($pOrc->get_doorsID(), my $pModuleID, my $pObjRecID);
	
	if (($cOrc->has_action($RC_ACTION_MOVE)) || ($cOrc->has_action($RC_ACTION_ADD)))
	{
		my $cBelow = $cOrc->get_below();
	
		if ($cBelow eq $pObjRecID)
		{
			my $aRcr = $pOrc->get_parent_rcrID();
			my $bRcr = $cOrc->get_parent_rcrID();
			die "Cannot approve RCR.  RCR $bRcr proposes to move or create an object below an object deleted or purged by RCR $aRcr";
		}
	}
}

sub checkDeleteOutlinkTargetConflict($;$)
{
	my $aOrc = shift;
	my $bOrc = shift;

	if ($bOrc->has_action($RC_ACTION_CREATE_OUTLINKS))
	{
		my $bOutlinkList = $bOrc->get_linkdata();
		ecpsSplitID($aOrc->get_doorsID(), my $aModuleId, my $aObjRecID);
	
		foreach my $bOutlink (@$bOutlinkList)
		{
			if (theSameOutlinkTarget($bOutlink, $aModuleId, $aObjRecID))
			{
				my $aRcr = $aOrc->get_parent_rcrID();
				my $bRcr = $bOrc->get_parent_rcrID();
				die "Unable to approve RCR due to conflict with RCR $bRcr.  RCR $bRcr has an outlink that targets an object that RCR $aRcr proposes to delete or purge\n";
			}		
		}
	}
}

sub checkCreateOutlinkConflict($;$)
{
	my $aOrc = shift;
	my $bOrc = shift;

	if ($aOrc->has_action($RC_ACTION_CREATE_OUTLINKS))
	{
		my $aOutlinkList = $aOrc->get_linkdata();
		ecpsSplitID($bOrc->get_doorsID(), my $bModuleId, my $bObjRecID);
	
		foreach my $aOutlink (@$aOutlinkList)
		{
			if (theSameOutlinkTarget($aOutlink, $bModuleId, $bObjRecID))
			{
				my $aRcr = $aOrc->get_parent_rcrID();
				my $bRcr = $bOrc->get_parent_rcrID();
				die "Unable to approve RCR due to conflict with RCR $bRcr.  RCR $aRcr modifies an outlink that targets an object that RCR $bRcr proposes to delete or purge\n";
			}
		}
	}
}

sub checkTwoMrcsForConflicts($;$;$;$;$;$)
{
	my $csapi = shift;
	my $user = shift;
	my $mrcOne = shift;
	my $mrcTwo = shift;
	my $rcrOne = shift;
	my $rcrTwo = shift;

	my $mrcOneActionsData = $mrcOne->getDataObjectByName("doorsChangeActions")->getValue();
	my $mrcTwoActionsData = $mrcTwo->getDataObjectByName("doorsChangeActions")->getValue();
	my $mrcOneActions = new ChangeSynergy::mrcActions($mrcOneActionsData);
	my $mrcTwoActions = new ChangeSynergy::mrcActions($mrcTwoActionsData);
	my $mrcOneObjRcs = $mrcOneActions->getAllObjectRcs();
	my $mrcTwoObjRcs = $mrcTwoActions->getAllObjectRcs();

	while (my ($aDoorsID, $aOrc) = each(%$mrcOneObjRcs))
	{
		if ($aOrc->get_doorsIsRejected() ne "TRUE")
		{
			while (my ($bDoorsID, $bOrc) = each(%$mrcTwoObjRcs))
			{
				if ($bOrc->get_doorsIsRejected() ne "TRUE")
				{
					ecpsSplitID($aOrc->get_doorsID(), my $aOrcModId, my $aOrcObjId);
					ecpsSplitID($bOrc->get_doorsID(), my $bOrcModId, my $bOrcObjId);
		
					if ($aDoorsID eq $bDoorsID)
					{
						my $aRcr = $aOrc->get_parent_rcrID();
						my $bRcr = $bOrc->get_parent_rcrID();

						#Both RCs are affecting the same DOORS object, there is a potential for conflict
						# --------------------------------
						# If a has purge....
						# --------------------------------

						if ($aOrc->has_action($RC_ACTION_PURGE))
						{
							if (!$bOrc->has_action($RC_ACTION_PURGE))
							{
								checkPurgeConflict($bOrc, $aRcr, $bRcr);
							}
						}
						# --------------------------
						# a has no purge,
						# If b has purge...
						# --------------------------
						elsif ($bOrc->has_action($RC_ACTION_PURGE))
						{
							checkPurgeConflict($aOrc, $aRcr, $bRcr);
						}
						# --------------------------
						# both a and b has no purge
						# If a has delete...
						# --------------------------
						elsif($aOrc->has_action($RC_ACTION_DELETE))
						{
							if (!$bOrc->has_action($RC_ACTION_DELETE))
							{
								checkDeleteConflict($bOrc, $aRcr, $bRcr);
							}
						}
						# --------------------------
						# a has no delete
						# If b has delete...
						# --------------------------
						elsif ($bOrc->has_action($RC_ACTION_DELETE))
						{
							checkDeleteConflict($aOrc, $aRcr, $bRcr);
						}
						# --------------------------
						# at this point, both a and b has no neither purge or delete
						# --------------------------
						else
						{
							# check move conflicts...
							checkMoveConflict($aOrc, $bOrc);
						}
					}
					elsif ($aOrcModId eq $bOrcModId)
					{
						#They are against different objects but the same module
						# -------------------------------------------
						# If a has purge.... check if b is moved/added under a
						# -------------------------------------------
						if ($aOrc->has_action($RC_ACTION_PURGE))
						{
							if (!$bOrc->has_action($RC_ACTION_PURGE))
							{
								checkParentConflict($aOrc, $bOrc, 0);
							}
						}
						# ----------------------------------------------
						# a has no purge,
						# If b has purge... check if a is moved/added under b
						# ----------------------------------------------
						elsif ($bOrc->has_action($RC_ACTION_PURGE))
						{
							checkParentConflict($aOrc, $bOrc, 1);
						}
						# -------------------------------------------
						# both a and b has no purge
						# If a has delete... check if b is moved/added under a when b is not deleted
						# -------------------------------------------
						elsif ($aOrc->has_action($RC_ACTION_DELETE))
						{
							if (!$bOrc->has_action($RC_ACTION_DELETE))
							{
								checkParentConflict($aOrc, $bOrc, 0);
							}
						}
						# ----------------------------------------------
						# a has no delete
						# If b has delete... check if a is moved/added under b when a is not deleted also
						# ----------------------------------------------
						elsif ($bOrc->has_action($RC_ACTION_DELETE))
						{
							checkParentConflict($aOrc, $bOrc, 1);
						}
					}
					# ===================================================================
					# The following tests have to be test no matter what... 
					# very expensive
					# ===================================================================

					# -----------------------------------------------------------------------
					# this check if we delete the object which is a outlink target of other RCR
					# -----------------------------------------------------------------------
					if (($aOrc->has_action($RC_ACTION_PURGE)) || ($aOrc->has_action($RC_ACTION_DELETE)))
					{
						checkDeleteOutlinkTargetConflict($aOrc, $bOrc);
					}

					# -----------------------------------------------------------------------
					# this check if other RCR delete the target object of created outlink
					# -----------------------------------------------------------------------
					if (($bOrc->has_action($RC_ACTION_PURGE)) || ($bOrc->has_action($RC_ACTION_DELETE)))
					{
						checkCreateOutlinkConflict($aOrc, $bOrc);
					}
				}
			}
			BORC: while (my ($bDoorsID, $bOrc) = each(%$mrcTwoObjRcs))
			{
				my $bParent_rcrID = $bOrc->get_parent_rcrID();
				my $aParent_rcrID = $aOrc->get_parent_rcrID();
				my $bIsApplied = $bOrc->get_doorsIsApplied();
				my $bIsRejected = $bOrc->get_doorsIsRejected();
				my $aRcId = $aOrc->get_rcID();
				my $bRcId = $bOrc->get_rcID();

				if ($bParent_rcrID eq $aParent_rcrID) { next BORC; }
				if ($bIsApplied eq "TRUE") { next BORC; }
				if ($bIsRejected eq "TRUE") { next BORC; }

				if ($aRcId ne $bRcId)
				{
					my $aDoorsId = $aOrc->get_doorsID();
					my $bDoorsId = $bOrc->get_doorsID();
					if ($aDoorsId eq $bDoorsID)
					{
						checkAttributeModificationConflict($aOrc, $bOrc);
					}
				}
			}
		}
	}
}

sub detectConflictsForOneMrc($;$;$;$;$)
{
	my $csapi = shift;
	my $user = shift;
	my $mrc = shift;
	my $to_state = shift;
	my $currentRcr = shift;

	my $mrcAttrList = "cvid|doorsID|doorsDBID|doorsIsApplied|doorsChangeActions|doors_object_path";

	#get some information from the MRC
	my $mrcCvid = $mrc->getDataObjectByName("cvid")->getValue();

	my $mrcWithAttrs = $csapi->GetObjectData($user, $mrcCvid, $mrcAttrList);

	my $mrcDoorsID = $mrcWithAttrs->getDataObjectByName("doorsID")->getValue();
	my $mrcDoorsDBID = $mrcWithAttrs->getDataObjectByName("doorsDBID")->getValue();
	my $origRcrNumber = $currentRcr->getDataObjectByName("problem_number")->getValue();
	my $mrcModulePath = $mrcWithAttrs->getDataObjectByName("doors_object_path")->getValue();
	print "Detecting conflicts for $origRcrNumber, module $mrcModulePath\n";

	#Find all the MRCs associated with other RCRs in approved state that happen to affect the same module
	my $queryString = "crstatus='" . $to_state . "' and has_dynamic_attrs_new(doorsID='$mrcDoorsID' and doorsDBID='$mrcDoorsDBID')";	
	my $report = "Data_Report_rcr_and_rc";
	my $otherRcrs = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);
	my $dataSize = $otherRcrs->getDataSize();
	print "Found $dataSize RCRs affecting same module\n";

	for (my $i=0; $i < $otherRcrs->getDataSize(); $i++)
	{
		my $otherRcr = $otherRcrs->getDataObject($i);
		my $childMrcs = $otherRcr->getChildData();
	
		for (my $j=0; $j<$childMrcs->getDataSize() ; $j++) 
		{
			my $childMrc = $childMrcs->getDataObject($j);
			my $childMrcDoorsID = $childMrc->getDataObjectByName("doorsID")->getValue();
			my $childMrcIsApplied = $childMrc->getDataObjectByName("doorsIsApplied")->getValue();
		
			if ($childMrcIsApplied ne "TRUE")
			{
				#This MRC is for the same module and is not yet applied, we need to check conflicts
				my $thisRcrNumber = $otherRcr->getDataObjectByName("problem_number")->getValue();
				print "Checking for conflicts between RCR $origRcrNumber and RCR $thisRcrNumber\n";
				my $mrc2 = $csapi->GetObjectData($user, $childMrc->getDataObjectByName("cvid")->getValue(), $mrcAttrList);
				checkTwoMrcsForConflicts($csapi,$user,$mrcWithAttrs,$mrc2,$currentRcr,$otherRcr);
			}
		}
	}
}


#Begin main body

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $
				      trigger->get_token(), $trigger->get_database());

#figure out what state the RCR is being transitioned to
my $to_state = $trigger->get_to_state();

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";
my $report = "Data_Report_rcr_and_rc";
my $curRcrData = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);

my $numRcrsReturned = $curRcrData->getDataSize;

if ($numRcrsReturned != 1)
{
	die "Unable to determine RCR being transitioned";
}

#Get the MRCs associated with the RCR
my $mrcs = $curRcrData->getDataObject(0)->getChildData();

#Loop through and look at each MRC
for (my $i = 0; $i < $mrcs->getDataSize(); $i++)
{
	my $mrc = $mrcs->getDataObject($i);
	my $mrcIsApplied = $mrc->getDataObjectByName("doorsIsApplied")->getValue();

	if ($mrcIsApplied ne "TRUE")
	{
		detectConflictsForOneMrc($csapi, $user, $mrc, $to_state, $curRcrData->getDataObject(0));
	}
}

#If we got here, there are no conflicts
exit(0);
