# <trigger>subscriptionRelationTrigger.pl<usage></usage>
#	<version>4.4</version>
#	<description>发送电子邮件至一个用户列表，这些用户当前在创建与某个 CR 的关联时已预订该 CR。
#               仅当创建的关联中涉及 CR 时，才会触发该触发器。仅当订户不是 CR 的修改者时，才会接收到一封电子邮件。
#                注：在使用该脚本前必须先编辑 pt.cfg 文件。有两个必须填写的配置项：SMTP_SERVER 和 FROM_EMAIL_ADDRESS。
#                     SMTP_SERVER 项用于定义 SMTP 服务器的地址，发送电子邮件时应使用该服务器。FROM_EMAIL_ADDRESS 项将用作为通过触发器发送的所有电子邮件中的“发件人”。
#   </description>
# </trigger>

use utf8;
use ChangeSynergy::emailHelper;
use strict;                         #Make sure all variables are defined before use.
use ChangeSynergy::TriggerParser;   #Used to parse the trigger XML file sent from ChangeSynergy.
use ChangeSynergy::csapi;           #Main module for the CS API.

#Create a new TriggerParser object, passing in the location of the trigger XML document.
#The trigger XML document is always the last parameter sent to a trigger.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Will die if smtp server or from address not set.
ChangeSynergy::EmailHelper::verifyMailData($trigger);

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object, password is not needed, uses the token
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "", $trigger->get_role(),
		   $trigger->get_token(), $trigger->get_database());

#Setup the EmailHelper module so it can share the csapi, user and trigger variables.
ChangeSynergy::EmailHelper::setup($csapi, $user, $trigger);

#Get the id's for the from and to objects of the relation.
my $fromObj = $trigger->get_from_object();
my $toObj   = $trigger->get_to_object();

#Get extra CR information from the server
my $crInfo = ChangeSynergy::EmailHelper::getCrDetails($fromObj, "problem_synopsis|crstatus");

#Create an instance of the Globals PERL module.
my $globals = new ChangeSynergy::Globals();

#Get the relation name and type.
my $relationType = $trigger->get_relation_type();
my $relationName = $trigger->get_relation_name();
my $relationAction= $trigger->get_relation_action();

#Get the subscriber email addresses, this list is comma delimited since Mail::Sender takes a comma
#delimited list.
my $subscribers = $trigger->get_subscriber_email_addresses();

#Use the ChangeSynergy::EmailHelper to open the email and get things going.
ChangeSynergy::EmailHelper::openEmail("", "", $subscribers, "IBM Rational Change subscription notification - CR " . $fromObj);

my $mainTableData = ChangeSynergy::EmailHelper::createCrInfoBlockData($crInfo, "Modifier");
ChangeSynergy::EmailHelper::sendStrippedTable("Change Request Information", $mainTableData);

my $sectionTitle;

if ($relationAction eq "CREATE_RELATION")
{
	$sectionTitle = "Relationship Created";
}
else
{
	$sectionTitle = "Relationship Deleted";
}

my $additionalTableData = createAdditionalBlockData($relationName);
ChangeSynergy::EmailHelper::sendStrippedTable($sectionTitle, $additionalTableData);

#Closes the email out and attaches the inline image.
ChangeSynergy::EmailHelper::closeEmail();

######################## Local Methods ########################

sub createAdditionalBlockData
{
	my $relationName = shift;

	my @loop_data = ();  # initialize an array to hold your loop
	my %row_data = ();

	$row_data{alias} = ChangeSynergy::EmailHelper::getAttributeLabel($relationName);
	$row_data{zebra} = 1;
	
	if ($relationType eq "CCM_PROBLEM_PROBLEM")
	{
		$row_data{value} = ChangeSynergy::EmailHelper::getCRLink($trigger, $fromObj) . " to " . 
						   ChangeSynergy::EmailHelper::getCRLink($trigger, $toObj);
	}
	elsif ($relationType eq "CCM_PROBLEM_TASK")
	{
		$row_data{value} = ChangeSynergy::EmailHelper::getCRLink($trigger, $fromObj) . " to " . 
						   ChangeSynergy::EmailHelper::getTaskLink($trigger, $toObj);
	}
	elsif ($relationType eq "CCM_PROBLEM_OBJECT")
	{
		$row_data{value} = ChangeSynergy::EmailHelper::getCRLink($trigger, $fromObj) . " to " . 
						   ChangeSynergy::EmailHelper::getObjectLink($trigger, $toObj);
	}
	elsif ($relationType eq "CCM_TASK_PROBLEM")
	{
		$row_data{value} = ChangeSynergy::EmailHelper::getTaskLink($trigger, $toObj) . " to " . 
						   ChangeSynergy::EmailHelper::getCRLink($trigger, $fromObj);
	}
	elsif ($relationType eq "CCM_OBJECT_PROBLEM")
	{
		$row_data{value} = ChangeSynergy::EmailHelper::getObjectLink($trigger, $fromObj) . " to " . 
						   ChangeSynergy::EmailHelper::getCRLink($trigger, $toObj);
	}

	push(@loop_data, \%row_data);

	return \@loop_data; 
}
